package com.application.GUI.Panes;

import com.application.DB.Constants;
import com.application.GUI.PopUpWindows.InputPopup;
import com.application.GUI.PopUpWindows.OutputPopup;
import com.application.GUI.ProgressBar.RingProgressIndicator;
import com.application.Main;
import javafx.application.Platform;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.Priority;
import javafx.scene.layout.VBox;

import java.util.logging.Logger;

import static com.application.DB.HelpingFunctions.isLoadedData;
import static com.application.DB.Settings.NUMBER_OF_SECONDS_LIVE_DATA;
import static com.application.GUI.LineChartFunctionality.getDataPointsXAxis;
import static com.application.GUI.LineChartFunctionality.getLiveData;
import static java.util.logging.Level.SEVERE;

/**
 * This class creates the sidebar
 *
 * @author Eilert Tunheim, Karin Pettersen, Mads Arnesen
 * @version 1.0.0
 */
public class CreateSideBar {

    public CreateSideBar(Main main) {
    }

    /**
     * This function creates the sidebar
     *
     * @return the finished sidebar
     */
    public VBox createSideBar() {

        // Creating a vbox
        VBox sideBarVBox = new VBox();

        // Defining label and text-field for tree species
        Label treeSpeciesLabel = new Label("Tree Species");
        treeSpeciesLabel.setId("sideBarLabelText");
        Main.setTreeSpeciesText(new TextField());
        Main.getTreeSpeciesText().setId("sideBarLabelText");
        Main.getTreeSpeciesText().setPromptText("No Input");
        Main.getTreeSpeciesText().setText(Constants.TREE_SPECIES);
        Main.getTreeSpeciesText().setEditable(false);

        // Defining label and text-field for dimensions
        Label dimensionsLabel = new Label("Width x Height");
        dimensionsLabel.setId("sideBarLabelText");
        Main.setDimensionsText(new TextField());
        Main.getDimensionsText().setId("sideBarLabelText");
        Main.getDimensionsText().setPromptText("No Input");
        Main.getDimensionsText().setText(Constants.DIMENSIONS);
        Main.getDimensionsText().setEditable(false);

        // Defining label and text-field for sawset
        Label sawsetLabel = new Label("Sawset");
        sawsetLabel.setId("sideBarLabelText");
        Main.setSawsetText(new TextField());
        Main.getSawsetText().setId("sideBarLabelText");
        Main.getSawsetText().setPromptText("No Input");
        Main.getSawsetText().setText(Constants.SAWSET);
        Main.getSawsetText().setEditable(false);

        // Defining label and text-field for moisture goal
        Label moistureGoalLabel = new Label("Moisture Goal");
        moistureGoalLabel.setId("sideBarLabelText");
        Main.setMoistureGoalText(new TextField());
        Main.getMoistureGoalText().setId("sideBarLabelText");
        Main.getMoistureGoalText().setPromptText("No Input");
        Main.getMoistureGoalText().setText(Constants.MOISTURE_GOAL);
        Main.getMoistureGoalText().setEditable(false);

        // Defining label and text-field timeleft
        Label timeLeftLabel = new Label("Time Left");
        timeLeftLabel.setId("sideBarLabelText");
        Main.setTimeLeftText(new TextField());
        Main.getTimeLeftText().setId("sideBarLabelText");
        Main.getTimeLeftText().setPromptText("Calculating...");
        Main.getTimeLeftText().setText(Constants.TIME_LEFT);
        Main.getTimeLeftText().setEditable(false);

        // Define buttons
        Button inputParametersButton = new Button("Input Parameters");
        inputParametersButton.setId("sideBarButtonInputParameters");
        inputParametersButton.setOnAction(e -> InputPopup.display());

        Button finishButton = new Button("Finish");
        finishButton.setId("sideBarButtonFinish");
        finishButton.setOnAction(e -> OutputPopup.displayOutputWindow());

        Button exitButton = new Button("Exit");
        exitButton.setId("sideBarButtonExit");
        exitButton.setOnAction(e -> Main.exitApplication());


        // Creating the circular progressbar
        RingProgressIndicator ringProgressIndicator = new RingProgressIndicator();
        ringProgressIndicator.setRingWidth(100);
        ringProgressIndicator.makeIndeterminate();
        ringProgressIndicator.setId("progressBar");

        // Defines thread for updating the circular progressbar
        class WorkerThread extends Thread{
            RingProgressIndicator rpi;
            int progress = 0;

            public WorkerThread(RingProgressIndicator rpi){
                this.rpi = rpi;
            }

            @Override
            public void run() {
                while (!Constants.IS_FINISHED) {

                    try {
                        ringProgressIndicator.makeIndeterminate();
                        Thread.sleep(500L * NUMBER_OF_SECONDS_LIVE_DATA);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }

                    while (isLoadedData()) {
                        try {
                            Thread.sleep(100);
                        } catch (InterruptedException e) {
                            e.printStackTrace();
                            Logger.getLogger(getClass().getName()).log(SEVERE, null, e);
                        }
                        Platform.runLater(() -> rpi.setProgress(progress));

                        progress = getLiveData().size()/ getDataPointsXAxis()*100;

                        if (progress > 100) {
                            break;
                        }
                    }
                }
            }
        }
        new WorkerThread(ringProgressIndicator).start();

        // Adds all the components to the VBox
        sideBarVBox.getChildren().addAll(ringProgressIndicator, treeSpeciesLabel, Main.getTreeSpeciesText(), dimensionsLabel, Main.getDimensionsText(),
                sawsetLabel, Main.getSawsetText(), moistureGoalLabel, Main.getMoistureGoalText(), timeLeftLabel, Main.getTimeLeftText(), inputParametersButton, finishButton, exitButton);

        VBox.setVgrow(sideBarVBox, Priority.ALWAYS);

        return sideBarVBox;
    }
}