package com.application.GUI.PopUpWindows;

import com.application.DB.HelpingFunctions;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.KeyCode;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;

import static com.application.DB.AccountHandler.*;
import static com.application.DB.HelpingFunctions.isValidInput;
import static com.application.GUI.Panes.CreateLogoBar.getLogin;

/**
 * This class handles all login functionality
 *
 * @author Eilert Tunheim, Karin Pettersen, Mads Arnesen
 * @version 1.0.0
 */
public class LoginPopup {

    // Defines input fields
    private static PasswordField PASSWORD_TEXT_FIELD = new PasswordField();
    private static TextField USERNAME_TEXT_FIELD = new TextField();

    /**
     * Tries to log in
     *
     * @return a boolean if an error should be thrown or not. True gives no error, false throws an error.
     */
    private static boolean loginButtonPressed(){
        try {
            getAccountInformation(getUsernameTextField().getText(), hashPassword(getPasswordTextField().getText()));
            if(getLogin().getText().equals("Admin")) {
                LoginPopup.adminPopup();
                return true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

        return !getLogin().getText().equals("Login");
    }


    /**
     * This function adds all components to the login window
     */
    public static void login(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Login window");

        Label userNameLabel = new Label("Username:");
        Label passwordLabel = new Label("Password:");

        Button closeButton = new Button("Close");
        Button loginButton = new Button("Login");
        getPasswordTextField().clear();

        // Closes the window if the close button is pressed
        closeButton.setOnAction(event -> window.close());

        // Handles login if enter is pressed on the keyboard in the password field
        getPasswordTextField().setOnKeyPressed( event -> {
                if (event.getCode() == KeyCode.ENTER) {
                    if(!isValidInput(getPasswordTextField().getText()) && !isValidInput(getUsernameTextField().getText())) {
                    if (loginButtonPressed()) {
                        window.close();
                    }
                }
            }
        });

        // Handles login if the login button is pressed
        loginButton.setOnAction(event -> {
            if(!isValidInput(getPasswordTextField().getText()) && !isValidInput(getUsernameTextField().getText())) {
                if (loginButtonPressed()) {
                    window.close();
                }
            }
        });

        // Adds all components
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(userNameLabel, getUsernameTextField(), passwordLabel, getPasswordTextField(), loginButton, closeButton);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    /**
     * This function handles admin window functionality
     */
    public static void adminPopup(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        // Information fields
        Label usernameLabel = new Label("Username: ");
        TextField usernameTextfield = new TextField();
        usernameTextfield.setText(HelpingFunctions.getUserName());
        usernameTextfield.setEditable(false);

        // Defines all buttons
        Button addUser = new Button("Add User");
        Button deleteUser = new Button("Delete User");
        Button logout = new Button("Logout");
        Button close = new Button("Close");

        // Handles functionality for each button
        addUser.setOnAction(event -> {
            adminAddUser();
        });
        deleteUser.setOnAction(event -> {
            adminDeleteUser();
        });
        logout.setOnAction(event -> {
            logout();
            window.close();
        });
        close.setOnAction(event -> window.close());

        // adds all components to the VBox
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(usernameLabel, usernameTextfield, addUser, deleteUser, logout, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    /**
     * This function adds a user to the database based on the input parameters given by the user
     */
    public static void adminAddUser(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        // Defines all labels
        Label firstNameLabel = new Label("First Name: ");
        Label lastNameLabel = new Label("Last Name: ");
        Label phoneNoLabel = new Label("Phone No: ");
        Label usernameLabel = new Label("Username: ");
        Label passwordFirstLabel = new Label("Password : ");
        Label passwordSecondLabel = new Label("Password Repeat: ");
        Label isAdminLabel = new Label("Is Admin: ");

        // Defines all input fields
        TextField firstNameTextField = new TextField();
        TextField lastNameTextField = new TextField();
        TextField phoneNoTextField = new TextField();
        TextField usernameTextField = new TextField();
        PasswordField passwordFirstField = new PasswordField();
        PasswordField passwordSecondField = new PasswordField();
        CheckBox isAdminBox = new CheckBox();
        isAdminBox.setSelected(false);

        // Defines all buttons
        Button close = new Button("Close");
        Button addUser = new Button("Add User");

        // Handles the functionality for each button
        close.setOnAction(event -> window.close());
        addUser.setOnAction(event -> {

            // Validate input parameters
            if(!isValidInput(firstNameTextField.getText()) &&
                    !isValidInput(lastNameTextField.getText()) &&
                    !isValidInput(phoneNoTextField.getText()) &&
                    !isValidInput(usernameTextField.getText()) &&
                    !isValidInput(passwordFirstField.getText()) &&
                    !isValidInput(passwordSecondField.getText())) {

            // If the passwords match each other, add the user, if not, display an error message
            if(passwordFirstField.getText().contentEquals(passwordSecondField.getText())){

                // Hashing the password if they match.
                String hashedPassword = hashPassword(passwordFirstField.getText());

                // Tries to add the user
                try {
                    if(addUser(firstNameTextField.getText(), lastNameTextField.getText(), phoneNoTextField.getText(),
                            usernameTextField.getText(), hashedPassword, isAdminBox.isSelected())){
                        NotificationPopUp.displayNotificationWindow("Successfully added user!");
                        window.close();
                    } else NotificationPopUp.displayNotificationWindow("That username already exist in the database!");
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } }else {
                NotificationPopUp.displayNotificationWindow("Passwords does not match!");
                passwordFirstField.clear();
                passwordSecondField.clear();
            }

        });

        // Adds all components to the VBox
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(firstNameLabel, firstNameTextField, lastNameLabel, lastNameTextField,
                                    phoneNoLabel, phoneNoTextField, usernameLabel, usernameTextField,
                                    passwordFirstLabel, passwordFirstField, passwordSecondLabel, passwordSecondField,
                                    isAdminLabel, isAdminBox, addUser, close);

        Scene scene = new Scene(layout, 500, 600);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    /**
     * This function handles deleting an account
     */
    public static void adminDeleteUser(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        // Defines the input field
        Label usernameLabel = new Label("Username: ");
        TextField usernameTextField = new TextField();

        // Defines all buttons
        Button close = new Button("Close");
        Button delete = new Button("Delete User");

        // Handle the functionality for each button
        close.setOnAction(event -> window.close());
        delete.setOnAction(event -> {
            // Validate the input parameter
            if(!isValidInput(usernameTextField.getText())) {
                try {
                    // Checks if the user is deleted
                    boolean results = deleteUser(usernameTextField.getText());
                    if (results) {
                        NotificationPopUp.displayNotificationWindow(usernameTextField.getText() + " was successfully deleted!");
                        window.close();
                    } else {
                        NotificationPopUp.displayNotificationWindow("Could not find username: " + usernameTextField.getText());
                        usernameTextField.clear();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
        });

        // Adds the components to the VBox
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(usernameLabel, usernameTextField, delete, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    /**
     * This function handles logout functionality
     */
    public static void logout(){
        getLogin().setText("Login");
        HelpingFunctions.setFirstName("");
        HelpingFunctions.setLastName("");
        HelpingFunctions.setIsAdmin(false);
        HelpingFunctions.setPhoneNo("");
        HelpingFunctions.setUserName("");
    }

    /**
     * This function handles popup screen for a normal user
     */
    public static void userPopup(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("User window");

        // Defines labels
        Label nameLabel = new Label("Name: ");
        Label phoneNoLabel = new Label("Phone no: ");

        // Defines information fields
        TextField nameTextfield = new TextField();
        TextField phoneNoTextField = new TextField();
        nameTextfield.setEditable(false);
        phoneNoTextField.setEditable(false);

        // Sets the values
        nameTextfield.setText(HelpingFunctions.getFirstName() + " " + HelpingFunctions.getLastName());
        phoneNoTextField.setText(HelpingFunctions.getPhoneNo());

        // Define buttons
        Button close = new Button("Close");
        Button logout = new Button("Logout");

        // Handle each button
        close.setOnAction(event -> window.close());
        logout.setOnAction(event -> {
            logout();
            window.close();
        });

        // Adds all components to the VBox
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(nameLabel, nameTextfield, phoneNoLabel, phoneNoTextField, logout, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    /**
     * This function hash the password using the SHA-512 algorithm
     *
     * @param password input parameter for the password
     * @return a 64-bit hashed string
     */
    public static String hashPassword(String password){
        try {
            MessageDigest messageDigest = MessageDigest.getInstance("SHA-512");

            assert messageDigest != null;
            messageDigest.update(password.getBytes(StandardCharsets.UTF_8));

            byte[] hashedPassword = messageDigest.digest();

            StringBuilder hashedPasswordString = new StringBuilder();

            for (byte b: hashedPassword) {
                hashedPasswordString.append(String.format("%02x",b));
            }

            return hashedPasswordString.toString();

        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static PasswordField getPasswordTextField() {
        return PASSWORD_TEXT_FIELD;
    }

    public static TextField getUsernameTextField() {
        return USERNAME_TEXT_FIELD;
    }
}
