package com.application.GUI.PopUpWindows;

import com.application.DB.Constants;
import com.application.DB.DB;
import com.application.DB.Settings;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;

import static com.application.DB.Constants.IS_FINISHED;
import static com.application.DB.Settings.MAX_USER_INPUT_CHARACTERS;
import static com.application.DB.HelpingFunctions.isValidInput;

/**
 * This class handles the output window for sending the resulted manual moisture levels
 *
 * @author Eilert Tunheim, Karin Pettersen, Mads Arnesen
 * @version 1.0.0
 */
public class OutputPopup {

    /**
     * This function handles the output window and it's functionality
     */
    public static void displayOutputWindow(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Moisture Check");

        // Top
        Label moistureCheckLabelHuge = new Label("Moisture Check");
        moistureCheckLabelHuge.setId("inputTop");
        moistureCheckLabelHuge.setAlignment(Pos.TOP_CENTER);

        // Center - Output field
        TextField moistureCheckTextField = new TextField();
        moistureCheckTextField.setId("inputLabel");
        moistureCheckTextField.setAlignment(Pos.CENTER);
        moistureCheckTextField.setPromptText("Please enter a value");



        // Bottom - Buttons
        Button countinueButton = new Button("Continue");
        countinueButton.setId("inputButtonStart");
        countinueButton.setAlignment(Pos.BOTTOM_CENTER);
        countinueButton.setOnAction(e -> {
                    try {

                        boolean err = false;

                        // Validates inputs
                        if(isValidInput(moistureCheckTextField.getText())){
                            err = true;
                        }

                        // Checks if there is an error
                        if(!err){
                            if(!moistureCheckTextField.getCharacters().toString().isEmpty() &&
                                    !moistureCheckTextField.getCharacters().toString().equals("Please enter a value")) {
                                Constants.STOP_TIME = null;
                                DB.pushManMoisture(moistureCheckTextField.getCharacters().toString());
                                Settings.NUMBER_OF_CHECKS++;
                                window.close();
                            } else {
                                NotificationPopUp.displayNotificationWindow("Please enter a value!");
                                moistureCheckTextField.setPromptText("Please enter a value");
                            }
                        } else {
                            moistureCheckTextField.setText("");
                            moistureCheckTextField.setPromptText("Please enter a value");
                        }



                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                });
        Button finishButton = new Button("Finish");
        finishButton.setId("inputButtonStart");
        finishButton.setAlignment(Pos.BOTTOM_CENTER);
        finishButton.setOnAction(e -> {
            try {

                boolean err = false;

                // Validates inputs
                if(isValidInput(moistureCheckTextField.getText())){
                    err = true;
                }

                // Checks if there is an error
                if(!err) {
                    if (!moistureCheckTextField.getCharacters().toString().isEmpty() &&
                            !moistureCheckTextField.getCharacters().toString().equals("Please enter a value")) {
                        IS_FINISHED = true;
                        Constants.STOP_TIME = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now());
                        DB.pushManMoisture(moistureCheckTextField.getCharacters().toString());
                        window.close();
                    } else {
                        NotificationPopUp.displayNotificationWindow("Please enter a value!");
                        moistureCheckTextField.setPromptText("Please enter a value");
                    }
                } else {
                    moistureCheckTextField.setText("");
                    moistureCheckTextField.setPromptText("Please enter a value");
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        });

        // Adds the components to the VBox
        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.setSpacing(10);
        layout.getChildren().addAll(moistureCheckLabelHuge,moistureCheckTextField,countinueButton,finishButton);

        Scene scene = new Scene(layout, 600, 500);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

}
